package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.apache.commons.lang3.StringUtils;
import pps.cloud.space.service.ISpaceInstitutionDetailCloudService;
import pps.core.base.entity.BaseDieselGeneratorEnt;
import pps.core.base.entity.BasePowerLineWellheadEnt;
import pps.core.base.entity.BaseWellheadEnt;
import pps.core.base.enums.BusinessError;
import pps.core.base.mapper.BasePowerLineWellheadMapper;
import pps.core.base.mapper.BaseWellheadMapper;
import pps.core.base.service.data.base_wellhead.*;
import pps.core.base.utils.ServiceUtil;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.BaseUtils;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;
import xstartup.helper.XTransactionHelper;

import java.util.List;
import java.util.Objects;


/**
 * 井口配置模块
 *
 * @author ZWT
 * @date 2023/09/08
 */
@XService
public class BaseWellheadService {

    @XText("新增")
    @XApiAnonymous
    @XApiPost
    public XServiceResult createBaseWellhead(XContext context, CreateBaseWellheadInput input) {
        return XTransactionHelper.begin(context, () -> {
            BaseWellheadMapper mapper = context.getBean(BaseWellheadMapper.class);
            BaseWellheadEnt entity = XCopyUtils.copyNewObject(input, BaseWellheadEnt.class);
            PpsUserSession session = context.getSession(PpsUserSession.class);
            BaseUtils.setBaseModelDefault(entity, session);
            mapper.insert(entity);
            return XServiceResult.OK;
        });
    }

    @XText("更新")
    @XApiAnonymous
    @XApiPost
    public XServiceResult updateBaseWellhead(XContext context, UpdateBaseWellheadInput input) {
        return XTransactionHelper.begin(context, () -> {
            BaseWellheadMapper mapper = context.getBean(BaseWellheadMapper.class);
            BaseWellheadEnt entity = this.selectOneByWellheadId(input.getId(), mapper);
            if (Objects.isNull(entity)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            XCopyUtils.copyObject(input, entity);
            PpsUserSession session = context.getSession(PpsUserSession.class);
            BaseUtils.setBaseModelDefault(entity, session);
            mapper.updateById(entity);
            return XServiceResult.OK;
        });
    }

    @XText("删除")
    @XApiAnonymous
    @XApiPost
    public XServiceResult deleteBaseWellhead(XContext context, DeleteBaseWellheadInput input) {
        String wellheadId = input.getId();
        if (this.checkWellheadIsLineReference(context, wellheadId)) {
            return XServiceResult.error(context, BusinessError.WellheadReferenceLine);
        }
        if (this.checkWellheadIsSpaceReference(context, wellheadId)) {
            return XServiceResult.error(context, BusinessError.WellheadReferenceSpace);
        }
        return XTransactionHelper.begin(context, () -> {
            BaseWellheadMapper mapper = context.getBean(BaseWellheadMapper.class);
            BaseWellheadEnt entity = this.selectOneByWellheadId(wellheadId, mapper);
            if (Objects.isNull(entity)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            entity = new BaseWellheadEnt();
            entity.setId(wellheadId);
            entity.setIsDeleted(BusinessConstant.ZERO);
            PpsUserSession session = context.getSession(PpsUserSession.class);
            BaseUtils.setBaseModelDefault(entity, session);
            mapper.updateById(entity);
            return XServiceResult.OK;
        });
    }

    @XText("根据ID获取详情")
    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetBaseWellheadOutput> getBaseWellhead(XContext context, GetBaseWellheadInput input) {
        BaseWellheadMapper mapper = context.getBean(BaseWellheadMapper.class);
        BaseWellheadEnt entity = this.selectOneByWellheadId(input.getId(), mapper);
        if (Objects.isNull(entity)) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseWellheadOutput output = XCopyUtils.copyNewObject(entity, GetBaseWellheadOutput.class);
        return XSingleResult.success(output);
    }

    @XText("分页查询")
    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryBaseWellheadOutput> queryBaseWellhead(XContext context, QueryBaseWellheadInput input) {
        String excludeIdFlag = input.getExcludeIdFlag();
        BaseWellheadMapper mapper = context.getBean(BaseWellheadMapper.class);
        QueryWrapper<BaseWellheadEnt> queryWrapper = new QueryWrapper<>();
        List<String> allListByOuId = ServiceUtil.getOrgIdsByPath(context, input.getOuId());
        queryWrapper.lambda()
                .notExists(StringUtils.equals("1", excludeIdFlag), "SELECT 1 FROM base_power_line_wellhead w WHERE w.ou_id = {0} AND base_wellhead.id = w.wellhead_id AND w.is_deleted = 1", input.getOuId())
                .eq(BaseWellheadEnt::getIsDeleted, BusinessConstant.ONE)
                .in(CollUtil.isNotEmpty(allListByOuId), BaseWellheadEnt::getOuId, allListByOuId)
                .orderByDesc(BaseWellheadEnt::getModifyTime);
        return XMapperHelper.query(mapper, input, queryWrapper, QueryBaseWellheadOutput.class);
    }

    /*-----------------------------------private-----------------------------------*/

    /**
     * 检查井口是否被线路引用
     *
     * @param context    上下文
     * @param wellheadId 井口ID
     * @return boolean
     */
    private boolean checkWellheadIsLineReference(XContext context, String wellheadId) {
        BasePowerLineWellheadMapper mapper = context.getBean(BasePowerLineWellheadMapper.class);
        Long count = mapper.selectCount(new LambdaQueryWrapper<BasePowerLineWellheadEnt>()
                .eq(BasePowerLineWellheadEnt::getWellheadId, wellheadId)
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
        );
        return count > 0;
    }

    /**
     * 检查井口是否被间开引用
     *
     * @param context    上下文
     * @param wellheadId 井口id
     * @return boolean
     */
    private boolean checkWellheadIsSpaceReference(XContext context, String wellheadId) {
        ISpaceInstitutionDetailCloudService contextBean = context.getBean(ISpaceInstitutionDetailCloudService.class);
        XSingleResult<Long> result = contextBean.getInstitutionWellheadCountByWellheadId(context, wellheadId);
        result.throwIfFail();
        return result.getResult() > 0;
    }

    /**
     * 通过井口ID查询井口信息
     *
     * @param wellheadId 井口id
     * @param mapper     映射器
     * @return {@link BaseDieselGeneratorEnt}
     */
    private BaseWellheadEnt selectOneByWellheadId(String wellheadId, BaseWellheadMapper mapper) {
        return mapper.selectOne(new LambdaQueryWrapper<BaseWellheadEnt>()
                .eq(BaseModel::getId, wellheadId)
        );
    }
}
