package pps.core.task.job;

import pps.cloud.space.service.ISpaceOptimizeShortCloudService;
import pps.core.task.constant.CronConstant;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.tool.XLoggerTool;
import xstartup.data.XServiceResult;
import xstartup.service.job.XJob;
import xstartup.service.job.annotation.XCronTrigger;

/**
 * 短期间开优化定时任务
 *
 * @author ZWT
 * @date 2023/09/20 11:39
 */
@XText("短期间开优化定时任务")
@XService
public class SpaceOptimizeShortJob implements XJob {

    /**
     * 每一天 23:30:00 执行
     *
     * @param xContext x上下文
     * @return {@link XServiceResult}
     */
    @XCronTrigger(value = CronConstant.END_OF_DAY)
    @Override
    public XServiceResult execute(XContext xContext) {
        XLoggerTool logger = xContext.getLogger();
        logger.info("------ SpaceOptimizeShortJob start:{}", System.currentTimeMillis());
        ISpaceOptimizeShortCloudService service = xContext.getBean(ISpaceOptimizeShortCloudService.class);
        XServiceResult result = service.optimizeShortJob(xContext);
        result.throwIfFail();
        logger.info("------ SpaceOptimizeShortJob end:{}", System.currentTimeMillis());
        return XServiceResult.OK;
    }
}
