package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.lang3.StringUtils;
import pps.cloud.base.service.IBasePowerLineCloudService;
import pps.cloud.base.service.data.base_power_line_wellhead.DynamicQueryBasePowerLineWellheadInput;
import pps.cloud.base.service.data.base_power_line_wellhead.DynamicQueryBasePowerLineWellheadViewOutput;
import pps.cloud.prediction.service.data.plant_predicted_power_data.DynamicQueryPlantPredictedPowerInput;
import pps.cloud.prediction.service.data.plant_predicted_power_data.DynamicQueryPlantPredictedPowerOutput;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.BaseUtils;
import pps.core.space.entity.*;
import pps.core.space.enums.BusinessError;
import pps.core.space.mapper.*;
import pps.core.space.service.data.SpaceOptimizeLineRelation;
import pps.core.space.service.data.space_institution_detail.*;
import pps.core.space.service.data.space_institution_duration.GetSpaceInstitutionDurationViewOutput;
import pps.core.space.service.data.space_institution_wellhead.CreateSpaceInstitutionWellheadInput;
import pps.core.space.service.data.space_institution_wellhead.GetSpaceInstitutionWellheadViewOutput;
import pps.core.space.service.data.space_institution_wellhead.UpdateSpaceInstitutionWellheadInput;
import pps.core.space.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.helper.XTransactionHelper;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 基础间开配置模块
 *
 * @author ZWT
 * @date 2023/09/05
 */
@XService
@XText("基础间开配置模块")
public class SpaceInstitutionDetailService extends SpaceOptimizeBaseService {

    /**
     * 基础间开配置--新增
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XApiAnonymous
    @XApiPost
    @XText("基础间开配置--新增")
    public XServiceResult createSpaceInstitutionDetail(XContext context, CreateSpaceInstitutionDetailInput input) {
        PpsUserSession session = context.getSession(PpsUserSession.class);
        return XTransactionHelper.begin(context, () -> {
            //新增间开配置
            SpaceInstitutionDetailMapper mapper = context.getBean(SpaceInstitutionDetailMapper.class);
            SpaceInstitutionDetailEnt entity = XCopyUtils.copyNewObject(input, SpaceInstitutionDetailEnt.class);
            BaseUtils.setBaseModelDefault(entity, session);
            mapper.insert(entity);
            String institutionId = entity.getId();
            List<CreateSpaceInstitutionWellheadInput> wellheadList = input.getWellheadList();
            if (CollUtil.isNotEmpty(wellheadList)) {
                this.saveInstitutionWellhead(context, session, institutionId, wellheadList);
            }
            return XServiceResult.OK;
        });
    }

    /**
     * 基础间开配置--修改
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XApiAnonymous
    @XApiPost
    @XText("基础间开配置--修改")
    public XServiceResult updateSpaceInstitutionDetail(XContext context, UpdateSpaceInstitutionDetailInput input) {
        String institutionId = input.getId();
        return XTransactionHelper.begin(context, () -> {
            SpaceInstitutionDetailMapper mapper = context.getBean(SpaceInstitutionDetailMapper.class);
            SpaceInstitutionDetailEnt entity = this.getInstitutionDetail(mapper, institutionId);
            if (Objects.isNull(entity)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            Integer isCurrentBasic = entity.getIsCurrentBasic();
            if (isCurrentBasic.equals(0)) {
                return XServiceResult.error(992, "基础制度无法修改");
            }
            PpsUserSession session = context.getSession(PpsUserSession.class);
            this.updateInstitutionDetail(context, session, mapper, institutionId, input, entity);
            return XServiceResult.OK;
        });
    }

    /**
     * 基础间开配置--删除
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XApiAnonymous
    @XApiPost
    @XText("基础间开配置--删除")
    public XServiceResult deleteSpaceInstitutionDetail(XContext context, DeleteSpaceInstitutionDetailInput input) {
        String institutionId = input.getId();
        return XTransactionHelper.begin(context, () -> {
            SpaceInstitutionDetailMapper mapper = context.getBean(SpaceInstitutionDetailMapper.class);
            SpaceInstitutionDetailEnt entity = this.getInstitutionDetail(mapper, institutionId);
            if (Objects.isNull(entity)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            Integer isCurrentBasic = entity.getIsCurrentBasic();
            if (isCurrentBasic.equals(0)) {
                return XServiceResult.error(992, "基础制度无法删除");
            }
            PpsUserSession session = context.getSession(PpsUserSession.class);
            SpaceInstitutionDurationMapper durationMapper = context.getBean(SpaceInstitutionDurationMapper.class);
            durationMapper.update(null, new LambdaUpdateWrapper<SpaceInstitutionDurationEnt>()
                    .eq(SpaceInstitutionDurationEnt::getInstitutionId, institutionId)
                    .set(BaseModel::getIsDeleted, BusinessConstant.ZERO)
                    .set(SpaceInstitutionDurationEnt::getModifyById, session.getId())
                    .set(SpaceInstitutionDurationEnt::getModifyByName, session.getUserName())
                    .set(SpaceInstitutionDurationEnt::getModifyTime, new Date())
            );
            SpaceInstitutionWellheadMapper wellheadMapper = context.getBean(SpaceInstitutionWellheadMapper.class);
            wellheadMapper.update(null, new LambdaUpdateWrapper<SpaceInstitutionWellheadEnt>()
                    .eq(SpaceInstitutionWellheadEnt::getInstitutionId, institutionId)
                    .set(BaseModel::getIsDeleted, BusinessConstant.ZERO)
                    .set(SpaceInstitutionWellheadEnt::getModifyById, session.getId())
                    .set(SpaceInstitutionWellheadEnt::getModifyByName, session.getUserName())
                    .set(SpaceInstitutionWellheadEnt::getModifyTime, new Date())
            );
            mapper.update(null, new LambdaUpdateWrapper<SpaceInstitutionDetailEnt>()
                    .eq(BaseModel::getId, institutionId)
                    .set(BaseModel::getIsDeleted, BusinessConstant.ZERO)
                    .set(SpaceInstitutionDetailEnt::getModifyById, session.getId())
                    .set(SpaceInstitutionDetailEnt::getModifyByName, session.getUserName())
                    .set(SpaceInstitutionDetailEnt::getModifyTime, new Date())
            );
            return XServiceResult.OK;
        });
    }

    /**
     * 基础间开配置--详情
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetSpaceInstitutionDetailViewOutput}>
     */
    @XApiAnonymous
    @XApiGet
    @XText("基础间开配置--详情")
    public XSingleResult<GetSpaceInstitutionDetailViewOutput> getSpaceInstitutionDetailView(XContext context, GetSpaceInstitutionDetailViewInput input) {
        String institutionId = input.getId();
        SpaceInstitutionDetailMapper mapper = context.getBean(SpaceInstitutionDetailMapper.class);
        SpaceInstitutionDetailEnt entity = this.getInstitutionDetail(mapper, institutionId);
        if (Objects.isNull(entity)) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetSpaceInstitutionDetailViewOutput output = XCopyUtils.copyNewObject(entity, GetSpaceInstitutionDetailViewOutput.class);
        SpaceInstitutionWellheadMapper wellheadMapper = context.getBean(SpaceInstitutionWellheadMapper.class);
        //查井口列表
        List<SpaceInstitutionWellheadEnt> wellheadEntList = wellheadMapper.selectList(new LambdaQueryWrapper<SpaceInstitutionWellheadEnt>()
                .eq(SpaceInstitutionWellheadEnt::getInstitutionId, institutionId)
                .orderByAsc(SpaceInstitutionWellheadEnt::getStartSeq)
        );
        DynamicQueryBasePowerLineWellheadInput wellheadInput = new DynamicQueryBasePowerLineWellheadInput();
        wellheadInput.setLineId(entity.getLineId());
        if (CollUtil.isNotEmpty(wellheadEntList)) {
            wellheadInput.setWellheadIds(wellheadEntList.stream()
                    .map(SpaceInstitutionWellheadEnt::getWellheadId)
                    .collect(Collectors.toList()));
        }
        List<DynamicQueryBasePowerLineWellheadViewOutput> powerLineWellheadList = this.getPowerLineWellheadList(context, wellheadInput);
        if (CollUtil.isNotEmpty(powerLineWellheadList)) {
            powerLineWellheadList.forEach(wellhead -> {
                SpaceInstitutionWellheadEnt ent = new SpaceInstitutionWellheadEnt();
                ent.setWellheadId(wellhead.getWellheadId());
                ent.setWellNumber(wellhead.getWellNumber());
                ent.setRunTypeKey(wellhead.getRunTypeKey());
                wellheadEntList.add(ent);
            });
        }
        if (CollUtil.isNotEmpty(wellheadEntList)) {
            List<GetSpaceInstitutionWellheadViewOutput> wellheadViewOutputs = XCopyUtils.copyNewList(wellheadEntList, GetSpaceInstitutionWellheadViewOutput.class);
            Map<String, String> runTypeMap = ServiceUtil.getDictMap(context, BusinessConstant.RUN_TYPE);
            Map<String, String> intervalTypeMap = ServiceUtil.getDictMap(context, BusinessConstant.INTERVAL_TYPE);
            wellheadViewOutputs.forEach(wellhead -> {
                wellhead.setRunTypeName(runTypeMap.get(wellhead.getRunTypeKey()));
                wellhead.setIntervalTypeName(intervalTypeMap.get(wellhead.getIntervalTypeKey()));
            });
            //查时段列表
            SpaceInstitutionDurationMapper durationMapper = context.getBean(SpaceInstitutionDurationMapper.class);
            List<SpaceInstitutionDurationEnt> durationEntList = durationMapper.selectList(new LambdaQueryWrapper<SpaceInstitutionDurationEnt>()
                    .eq(SpaceInstitutionDurationEnt::getInstitutionId, institutionId)
                    .orderByAsc(SpaceInstitutionDurationEnt::getOpenWellTime)
            );
            if (CollUtil.isNotEmpty(durationEntList)) {
                List<GetSpaceInstitutionDurationViewOutput> durationViewOutputList = XCopyUtils.copyNewList(durationEntList, GetSpaceInstitutionDurationViewOutput.class);
                Map<String, List<GetSpaceInstitutionDurationViewOutput>> collect = durationViewOutputList.stream()
                        .collect(Collectors.groupingBy(GetSpaceInstitutionDurationViewOutput::getWellheadId));
                wellheadViewOutputs.forEach(wellhead -> {
                    String wellheadId = wellhead.getWellheadId();
                    if (collect.containsKey(wellheadId)) {
                        wellhead.setDurationList(collect.get(wellheadId));
                    } else {
                        wellhead.setDurationList(new ArrayList<>(0));
                    }
                });
            }
            output.setWellheadList(wellheadViewOutputs);
        } else {
            output.setWellheadList(new ArrayList<>(0));
        }
        output.setOuName(ServiceUtil.getSysOrgNameById(context, output.getOuId()));
        output.setGridTypeName(ServiceUtil.getDictMap(context, BusinessConstant.GRID_TYPE).get(output.getGridTypeKey()));
        output.setLineName(ServiceUtil.getPowerLineNameById(context, output.getLineId()));
        return XSingleResult.success(output);
    }

    /**
     * 基础间开配置--分页列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult}<{@link QuerySpaceInstitutionDetailViewOutput}>
     */
    @XApiAnonymous
    @XApiGet
    @XText("基础间开配置--分页列表")
    public XPageResult<QuerySpaceInstitutionDetailViewOutput> querySpaceInstitutionDetailView(XContext context, QuerySpaceInstitutionDetailViewInput input) {
        String lineId = input.getLineId();
        String institutionName = input.getInstitutionName();
        SpaceInstitutionDetailMapper mapper = context.getBean(SpaceInstitutionDetailMapper.class);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<SpaceInstitutionDetailEnt> detailEntList = mapper.selectList(new LambdaQueryWrapper<SpaceInstitutionDetailEnt>()
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                .eq(SpaceInstitutionDetailEnt::getOuId, input.getOuId())
                .eq(StringUtils.isNotBlank(lineId), SpaceInstitutionDetailEnt::getLineId, lineId)
                .like(StringUtils.isNotBlank(institutionName), SpaceInstitutionDetailEnt::getInstitutionName, institutionName)
        );
        PageInfo<SpaceInstitutionDetailEnt> pageInfo = new PageInfo<>(detailEntList);
        List<QuerySpaceInstitutionDetailViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySpaceInstitutionDetailViewOutput.class);
        if (CollUtil.isNotEmpty(outputs)) {
            Map<String, String> gridTypeMap = ServiceUtil.getDictMap(context, BusinessConstant.GRID_TYPE);
            Map<String, String> lineNameMap = ServiceUtil.getPowerLineNameMap(context, outputs.stream()
                    .map(QuerySpaceInstitutionDetailViewOutput::getLineId)
                    .distinct()
                    .collect(Collectors.toList())
            );
            String orgName = ServiceUtil.getSysOrgNameById(context, input.getOuId());
            outputs.forEach(output -> {
                output.setOuName(orgName);
                output.setGridTypeName(gridTypeMap.get(output.getGridTypeKey()));
                output.setLineName(lineNameMap.get(output.getLineId()));
                output.setIsCurrentBasicValue(output.getIsCurrentBasic().equals(0) ? "是" : "否");
            });
        }
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    /**
     * 基础间开配置--设为基础制度
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XApiAnonymous
    @XApiPost
    @XText("基础间开配置--设为基础制度")
    public XServiceResult updateBasicInstitution(XContext context, UpdateSpaceInstitutionDetailInput input) {
        String ouId = input.getOuId();
        String lineId = input.getLineId();
        String institutionId = input.getId();
        return XTransactionHelper.begin(context, () -> {
            SpaceInstitutionDetailMapper mapper = context.getBean(SpaceInstitutionDetailMapper.class);
            SpaceInstitutionDetailEnt detail = this.getInstitutionDetail(mapper, institutionId);
            if (Objects.isNull(detail)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            Integer isCurrentBasic = detail.getIsCurrentBasic();
            if (isCurrentBasic.equals(BusinessConstant.ZERO)) {
                return XServiceResult.error(context, BusinessError.RepeatSetting);
            }
            //关闭当前基础制度
            mapper.update(null, new LambdaUpdateWrapper<SpaceInstitutionDetailEnt>()
                    .eq(SpaceInstitutionDetailEnt::getOuId, ouId)
                    .eq(SpaceInstitutionDetailEnt::getLineId, lineId)
                    .eq(SpaceInstitutionDetailEnt::getIsCurrentBasic, BusinessConstant.ZERO)
                    .set(SpaceInstitutionDetailEnt::getIsCurrentBasic, BusinessConstant.ONE)
            );
            //设定当前制度为基础制度
            mapper.update(null, new LambdaUpdateWrapper<SpaceInstitutionDetailEnt>()
                    .eq(BaseModel::getId, institutionId)
                    .set(SpaceInstitutionDetailEnt::getIsCurrentBasic, BusinessConstant.ZERO)
            );
            //长期/中短期制度优化初始化
            this.institutionOptimizeInitialize(context, detail);
            return XServiceResult.OK;
        });
    }

    /**
     * 校准历史--重新优化
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XApiAnonymous
    @XApiPost
    @XText("校准历史--重新优化")
    public XServiceResult anewOptimizeInstitution(XContext context, UpdateSpaceInstitutionDetailInput input) {
        PpsUserSession session = context.getSession(PpsUserSession.class);
        String institutionId = input.getId();
        SpaceInstitutionDetailMapper mapper = context.getBean(SpaceInstitutionDetailMapper.class);
        SpaceInstitutionDetailEnt detail = this.getInstitutionDetail(mapper, institutionId);
        if (Objects.isNull(detail)) {
            return XServiceResult.error(context, XError.NotFound);
        }
        return XTransactionHelper.begin(context, () -> {
            this.updateInstitutionDetail(context, session, mapper, institutionId, input, detail);
            //制度优化
            this.institutionOptimizeInitialize(context, detail);
            return XServiceResult.OK;
        });
    }

    /**
     * 制度优化
     *
     * @param context 上下文
     * @param detail  细节
     */
    private void institutionOptimizeInitialize(XContext context, SpaceInstitutionDetailEnt detail) {
        String institutionId = detail.getId();
        //取制度开始时间,判断是否可以执行
        Date institutionStartDate = detail.getInstitutionStartDate();
        DateTime currentTime = DateUtil.date();
        if (currentTime.compareTo(institutionStartDate) >= 0) {
            List<SpaceOptimizePeriodDTO> longPeriodDTOList = new ArrayList<>(32);
            List<SpaceOptimizeWellheadDTO> longWellheadDTOList = new ArrayList<>(64);
            List<SpaceOptimizeDurationDTO> longDurationDTOList = new ArrayList<>(128);
            List<SpaceOptimizeDurationDTO> longUnOptimizeDurationList = new ArrayList<>(128);
            List<SpaceOptimizePeriodDTO> midPeriodDTOList = new ArrayList<>(32);
            List<SpaceOptimizeWellheadDTO> midWellheadDTOList = new ArrayList<>(64);
            List<SpaceOptimizeDurationDTO> midDurationDTOList = new ArrayList<>(128);
            List<SpaceOptimizeDurationDTO> midUnOptimizeDurationList = new ArrayList<>(128);
            DateTime endOfMonth = DateUtil.endOfMonth(currentTime);
            DateTime endOfWeek = DateUtil.endOfWeek(currentTime);
            String lineId = detail.getLineId();
            //取年份
            int year = currentTime.year();
            //月份要加一
            int monthNum = currentTime.month() + 1;
            String month;
            if (monthNum < BusinessConstant.TEN) {
                month = "0" + monthNum;
            } else {
                month = String.valueOf(monthNum);
            }
            SpaceOptimizeLineRelation wellheadAndPlant = super.getWellheadAndPlant(context, ListUtil.toList(detail));
            //长期间开优化
            List<DynamicQueryPlantPredictedPowerOutput> longAvgPowerList = super.getAveragePowerGenerationListByPlantIds(context,
                    DynamicQueryPlantPredictedPowerInput.builder()
                            .plantIds(wellheadAndPlant.getPlantIdsByLineIdMap().get(lineId))
                            .yearTime(String.valueOf(year))
                            .monthTime(month)
                            .build()
            );
            this.institutionalOptimization(context, longPeriodDTOList, longWellheadDTOList, longDurationDTOList, longUnOptimizeDurationList, wellheadAndPlant,
                    longAvgPowerList, detail, BaseUtils.getExecutionCycleForMonth(currentTime), endOfMonth, monthNum);
            //中短期间开优化
            List<DynamicQueryPlantPredictedPowerOutput> midAvgPowerList = super.getAveragePowerGenerationListByPlantIds(context,
                    DynamicQueryPlantPredictedPowerInput.builder()
                            .plantIds(wellheadAndPlant.getPlantIdsByLineIdMap().get(lineId))
                            .startTime(DateUtil.beginOfWeek(currentTime).toString(BusinessConstant.DATE_FORMAT_DAY))
                            .endTime(DateUtil.beginOfWeek(DateUtil.nextWeek()).toString(BusinessConstant.DATE_FORMAT_DAY))
                            .build()
            );
            this.institutionalOptimization(context, midPeriodDTOList, midWellheadDTOList, midDurationDTOList, midUnOptimizeDurationList, wellheadAndPlant,
                    midAvgPowerList, detail, BaseUtils.getExecutionCycleForWeek(currentTime), endOfWeek, monthNum);
            //逻辑删除已创建的长期优化数据
            SpaceOptimizeLongPeriodMapper longPeriodMapper = context.getBean(SpaceOptimizeLongPeriodMapper.class);
            List<SpaceOptimizeLongPeriodEnt> longPeriodEntList = longPeriodMapper.selectList(new LambdaQueryWrapper<SpaceOptimizeLongPeriodEnt>()
                    .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                    .eq(SpaceOptimizeLongPeriodEnt::getOptimizeState, BusinessConstant.ZERO)
                    .eq(SpaceOptimizeLongPeriodEnt::getInstitutionId, institutionId)
                    .eq(SpaceOptimizeLongPeriodEnt::getLineId, lineId)
                    .eq(SpaceOptimizeLongPeriodEnt::getOptimizeDeadline, endOfMonth)
            );
            if (CollUtil.isNotEmpty(longPeriodEntList)) {
                List<String> longIds = longPeriodEntList.stream()
                        .map(BaseModel::getId)
                        .collect(Collectors.toList());
                longPeriodMapper.update(null, new LambdaUpdateWrapper<SpaceOptimizeLongPeriodEnt>()
                        .set(BaseModel::getIsDeleted, BusinessConstant.ZERO)
                        .in(BaseModel::getId, longIds)
                );
                //删除关联数据
                SpaceOptimizeLongWellheadMapper longWellheadMapper = context.getBean(SpaceOptimizeLongWellheadMapper.class);
                longWellheadMapper.update(null, new LambdaUpdateWrapper<SpaceOptimizeLongWellheadEnt>()
                        .set(BaseModel::getIsDeleted, BusinessConstant.ZERO)
                        .in(SpaceOptimizeLongWellheadEnt::getLongPeriodId, longIds)
                );
                SpaceOptimizeLongDurationMapper longDurationMapper = context.getBean(SpaceOptimizeLongDurationMapper.class);
                longDurationMapper.update(null, new LambdaUpdateWrapper<SpaceOptimizeLongDurationEnt>()
                        .set(BaseModel::getIsDeleted, BusinessConstant.ZERO)
                        .in(SpaceOptimizeLongDurationEnt::getLongPeriodId, longIds)
                );
            }
            //逻辑删除已创建的中短期优化数据
            SpaceOptimizeMidPeriodMapper midPeriodMapper = context.getBean(SpaceOptimizeMidPeriodMapper.class);
            List<SpaceOptimizeMidPeriodEnt> midPeriodEntList = midPeriodMapper.selectList(new LambdaQueryWrapper<SpaceOptimizeMidPeriodEnt>()
                    .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                    .eq(SpaceOptimizeMidPeriodEnt::getOptimizeState, BusinessConstant.ZERO)
                    .eq(SpaceOptimizeMidPeriodEnt::getInstitutionId, institutionId)
                    .eq(SpaceOptimizeMidPeriodEnt::getLineId, lineId)
                    .eq(SpaceOptimizeMidPeriodEnt::getOptimizeDeadline, endOfWeek)
            );
            if (CollUtil.isNotEmpty(midPeriodEntList)) {
                List<String> midIds = midPeriodEntList.stream()
                        .map(BaseModel::getId)
                        .collect(Collectors.toList());
                midPeriodMapper.update(null, new LambdaUpdateWrapper<SpaceOptimizeMidPeriodEnt>()
                        .set(BaseModel::getIsDeleted, BusinessConstant.ZERO)
                        .in(BaseModel::getId, midIds)
                );
                //删除关联数据
                SpaceOptimizeMidWellheadMapper longWellheadMapper = context.getBean(SpaceOptimizeMidWellheadMapper.class);
                longWellheadMapper.update(null, new LambdaUpdateWrapper<SpaceOptimizeMidWellheadEnt>()
                        .set(BaseModel::getIsDeleted, BusinessConstant.ZERO)
                        .in(SpaceOptimizeMidWellheadEnt::getMidPeriodId, midIds)
                );
                SpaceOptimizeMidDurationMapper longDurationMapper = context.getBean(SpaceOptimizeMidDurationMapper.class);
                longDurationMapper.update(null, new LambdaUpdateWrapper<SpaceOptimizeMidDurationEnt>()
                        .set(BaseModel::getIsDeleted, BusinessConstant.ZERO)
                        .in(SpaceOptimizeMidDurationEnt::getMidPeriodId, midIds)
                );
            }
            //创建长期优化数据
            int size;
            SpaceOptimizeViewMapper optimizeViewMapper = context.getBean(SpaceOptimizeViewMapper.class);
            if (CollUtil.isNotEmpty(longPeriodDTOList)) {
                size = longPeriodDTOList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizePeriodDTO>> subList = BaseUtils.getSubList(longPeriodDTOList);
                    subList.forEach(b -> optimizeViewMapper.longPeriodBatchInsertList(b));
                } else {
                    optimizeViewMapper.longPeriodBatchInsertList(longPeriodDTOList);
                }
            }
            if (CollUtil.isNotEmpty(longWellheadDTOList)) {
                size = longWellheadDTOList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizeWellheadDTO>> subList = BaseUtils.getSubList(longWellheadDTOList);
                    subList.forEach(b -> optimizeViewMapper.longWellheadBatchInsertList(b));
                } else {
                    optimizeViewMapper.longWellheadBatchInsertList(longWellheadDTOList);
                }
            }
            if (CollUtil.isNotEmpty(longDurationDTOList)) {
                size = longDurationDTOList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizeDurationDTO>> subList = BaseUtils.getSubList(longDurationDTOList);
                    subList.forEach(b -> optimizeViewMapper.longDurationBatchInsertList(b));
                } else {
                    optimizeViewMapper.longDurationBatchInsertList(longDurationDTOList);
                }
            }
            if (CollUtil.isNotEmpty(longUnOptimizeDurationList)) {
                size = longUnOptimizeDurationList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizeDurationDTO>> subList = BaseUtils.getSubList(longUnOptimizeDurationList);
                    subList.forEach(b -> optimizeViewMapper.longDurationBatchInsertList(b));
                } else {
                    optimizeViewMapper.longDurationBatchInsertList(longUnOptimizeDurationList);
                }
            }
            //创建中短期优化数据
            if (CollUtil.isNotEmpty(midPeriodDTOList)) {
                size = midPeriodDTOList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizePeriodDTO>> subList = BaseUtils.getSubList(midPeriodDTOList);
                    subList.forEach(b -> optimizeViewMapper.midPeriodBatchInsertList(b));
                } else {
                    optimizeViewMapper.midPeriodBatchInsertList(midPeriodDTOList);
                }
            }
            if (CollUtil.isNotEmpty(midWellheadDTOList)) {
                size = midWellheadDTOList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizeWellheadDTO>> subList = BaseUtils.getSubList(midWellheadDTOList);
                    subList.forEach(b -> optimizeViewMapper.midWellheadBatchInsertList(b));
                } else {
                    optimizeViewMapper.midWellheadBatchInsertList(midWellheadDTOList);
                }
            }
            if (CollUtil.isNotEmpty(midDurationDTOList)) {
                size = midDurationDTOList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizeDurationDTO>> subList = BaseUtils.getSubList(midDurationDTOList);
                    subList.forEach(b -> optimizeViewMapper.midDurationBatchInsertList(b));
                } else {
                    optimizeViewMapper.midDurationBatchInsertList(midDurationDTOList);
                }
            }
            if (CollUtil.isNotEmpty(midUnOptimizeDurationList)) {
                size = midUnOptimizeDurationList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizeDurationDTO>> subList = BaseUtils.getSubList(midUnOptimizeDurationList);
                    subList.forEach(b -> optimizeViewMapper.midDurationBatchInsertList(b));
                } else {
                    optimizeViewMapper.midDurationBatchInsertList(midUnOptimizeDurationList);
                }
            }
        }
    }


    /**
     * 间开优化历史--分页列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult}<{@link QueryOptimizeHistoryViewOutput}>
     */
    @XApiAnonymous
    @XApiGet
    @XText("间开优化历史--分页列表")
    public XPageResult<QueryOptimizeHistoryViewOutput> queryOptimizeHistoryView(XContext context, QueryOptimizeHistoryViewInput input) {
        SpaceInstitutionDetailViewMapper mapper = context.getBean(SpaceInstitutionDetailViewMapper.class);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<SpaceInstitutionDetailView> historyList = mapper.selectOptimizeHistoryList(SpaceInstitutionDetailView.builder()
                .ouId(input.getOuId())
                .build()
        );
        PageInfo<SpaceInstitutionDetailView> pageInfo = new PageInfo<>(historyList);
        List<QueryOptimizeHistoryViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryOptimizeHistoryViewOutput.class);
        if (CollUtil.isNotEmpty(outputs)) {
            Map<String, String> lineNameMap = ServiceUtil.getPowerLineNameMap(context, outputs.stream()
                    .map(QueryOptimizeHistoryViewOutput::getLineId)
                    .distinct()
                    .collect(Collectors.toList())
            );
            outputs.forEach(o -> o.setLineName(lineNameMap.get(o.getLineId())));
        }
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    /*-----------------------------------private-----------------------------------*/

    /**
     * 条件查询输电线路井口列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link List}<{@link DynamicQueryBasePowerLineWellheadViewOutput}>
     */
    private List<DynamicQueryBasePowerLineWellheadViewOutput> getPowerLineWellheadList(XContext context, DynamicQueryBasePowerLineWellheadInput input) {
        IBasePowerLineCloudService lineService = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLineWellheadViewOutput> result = lineService.getPowerLineWellheadList(context, input);
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 保存策略关联信息
     *
     * @param session       一场
     * @param institutionId 机构id
     * @param wellheadList  井口一览表
     */
    private void saveInstitutionWellhead(XContext context, PpsUserSession session, String institutionId, List wellheadList) {
        SpaceInstitutionWellheadViewMapper wellheadViewMapper = context.getBean(SpaceInstitutionWellheadViewMapper.class);
        SpaceInstitutionDurationViewMapper durationViewMapper = context.getBean(SpaceInstitutionDurationViewMapper.class);
        List<SpaceInstitutionDurationView> durationViewList = new ArrayList<>(128);
        List<SpaceInstitutionWellheadView> wellheadViewList = new ArrayList<>(32);
        List<SpaceInstitutionDurationView> durationList;
        for (SpaceInstitutionWellheadView wellhead : (List<SpaceInstitutionWellheadView>) XCopyUtils.copyNewList(wellheadList, SpaceInstitutionWellheadView.class)) {
            wellhead.setInstitutionId(institutionId);
            BaseUtils.setBaseModelDefault(wellhead, session);
            wellhead.setId(BaseUtils.getUUID());
            wellheadViewList.add(wellhead);
            durationList = wellhead.getDurationList();
            if (CollUtil.isNotEmpty(durationList)) {
                durationList.forEach(d -> {
                    d.setWellheadId(wellhead.getWellheadId());
                    d.setConfigId(wellhead.getId());
                    d.setInstitutionId(institutionId);
                    BaseUtils.setBaseModelDefault(d, session);
                    d.setId(BaseUtils.getUUID());
                    durationViewList.add(d);
                });
            }
        }
        wellheadViewMapper.batchInsertList(wellheadViewList);
        if (CollUtil.isNotEmpty(durationViewList)) {
            durationViewMapper.batchInsertList(durationViewList);
        }
    }

    /**
     * 获取间开详情
     *
     * @param mapper        映射器
     * @param institutionId 机构id
     * @return {@link SpaceInstitutionDetailEnt}
     */
    private SpaceInstitutionDetailEnt getInstitutionDetail(SpaceInstitutionDetailMapper mapper, String institutionId) {
        return mapper.selectOne(new LambdaQueryWrapper<SpaceInstitutionDetailEnt>()
                .eq(BaseModel::getId, institutionId)
        );
    }

    /**
     * 更新间开制度及关联信息
     *
     * @param context       上下文
     * @param session       一场
     * @param mapper        映射器
     * @param institutionId 机构id
     * @param input         输入
     * @param entity        实体
     */
    private void updateInstitutionDetail(XContext context, PpsUserSession session, SpaceInstitutionDetailMapper mapper, String institutionId,
                                         UpdateSpaceInstitutionDetailInput input, SpaceInstitutionDetailEnt entity) {
        List<UpdateSpaceInstitutionWellheadInput> wellheadList = input.getWellheadList();
        if (CollUtil.isNotEmpty(wellheadList)) {
            //删除之前关联的信息
            SpaceInstitutionWellheadMapper wellheadMapper = context.getBean(SpaceInstitutionWellheadMapper.class);
            wellheadMapper.delete(new LambdaQueryWrapper<SpaceInstitutionWellheadEnt>()
                    .eq(SpaceInstitutionWellheadEnt::getInstitutionId, institutionId)
            );
            SpaceInstitutionDurationMapper durationMapper = context.getBean(SpaceInstitutionDurationMapper.class);
            durationMapper.delete(new LambdaQueryWrapper<SpaceInstitutionDurationEnt>()
                    .eq(SpaceInstitutionDurationEnt::getInstitutionId, institutionId)
            );
            //重新添加关联信息
            this.saveInstitutionWellhead(context, session, institutionId, wellheadList);
        }
        XCopyUtils.copyObject(input, entity);
        BaseUtils.setBaseModelDefault(entity, session);
        mapper.updateById(entity);
    }


    /**
     * 制度优化
     *
     * @param context                上下文
     * @param periodDTOList          时期数据主义者
     * @param wellheadDTOList        井口dtolist
     * @param durationDTOList        持续时间dtolist
     * @param unOptimizeDurationList 取消优化工期列表
     * @param avgPowerList           平均功率列表
     * @param detail                 细节
     * @param executionCycle         执行周期
     * @param optimizeDeadline       优化截止日期
     * @param monthNum               月份
     */
    private void institutionalOptimization(XContext context, List<SpaceOptimizePeriodDTO> periodDTOList, List<SpaceOptimizeWellheadDTO> wellheadDTOList,
                                           List<SpaceOptimizeDurationDTO> durationDTOList, List<SpaceOptimizeDurationDTO> unOptimizeDurationList,
                                           SpaceOptimizeLineRelation wellheadAndPlant, List<DynamicQueryPlantPredictedPowerOutput> avgPowerList, SpaceInstitutionDetailEnt detail,
                                           String executionCycle, Date optimizeDeadline, int monthNum) {
        String detailId = detail.getId();
        String lineId = detail.getLineId();
        List<SpaceInstitutionWellheadView> spaceWellheadList = wellheadAndPlant.getSpaceWellheadList();
        SpaceInstitutionDurationMapper durationMapper = context.getBean(SpaceInstitutionDurationMapper.class);
        //创建记录
        String periodId = super.createOptimizePeriod(periodDTOList, detailId, lineId, executionCycle, optimizeDeadline);
        switch (detail.getGridTypeKey()) {
            //并网型优化
            case "1":
                super.gridConnectedOptimization(context, durationMapper, wellheadDTOList, durationDTOList, unOptimizeDurationList,
                        spaceWellheadList, avgPowerList, detail, monthNum, detailId, periodId, lineId);
                break;
            //离网型优化
            case "0":
                break;
            default:
                //电网类型不存在
        }
    }
}
