package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import org.apache.commons.lang3.StringUtils;
import pps.cloud.base.service.data.base_power_line_storage.DynamicQueryBasePowerLineStorageViewOutput;
import pps.cloud.prediction.service.data.plant_predicted_power_data.DynamicQueryPlantPredictedPowerInput;
import pps.cloud.prediction.service.data.plant_predicted_power_data.DynamicQueryPlantPredictedPowerOutput;
import pps.cloud.prediction.service.data.storage_predicted_power_data.DynamicQueryStoragePredictedPowerInput;
import pps.cloud.prediction.service.data.storage_predicted_power_data.DynamicQueryStoragePredictedPowerOutput;
import pps.cloud.space.service.ISpaceOptimizeLongCloudService;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.utils.BaseUtils;
import pps.core.space.entity.*;
import pps.core.space.mapper.SpaceInstitutionDurationMapper;
import pps.core.space.mapper.SpaceOptimizeViewMapper;
import pps.core.space.service.data.SpaceOptimizeLineRelation;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.data.XServiceResult;
import xstartup.helper.XTransactionHelper;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * 长期间开优化Cloud模块
 *
 * @author ZWT
 * @date 2023/09/13 14:14
 */
@XService
public class SpaceOptimizeLongCloudServiceImpl extends SpaceOptimizeBaseService implements ISpaceOptimizeLongCloudService {

    /**
     * 长期间开优化Cloud模块--定时任务
     *
     * @param context 上下文
     * @return {@link XServiceResult}
     */
    @Override
    public XServiceResult optimizeLongJob(XContext context) {
        //取当前时间
        DateTime date = DateUtil.date();
        //取生效中的基础间开
        List<SpaceInstitutionDetailEnt> detailEntList = super.getEffectiveSpaceInstitution(context, date.toString(BusinessConstant.DATE_FORMAT_DAY));
        SpaceOptimizeLineRelation relation = super.getWellheadAndPlant(context, detailEntList);
        List<SpaceInstitutionWellheadView> spaceWellheadList = relation.getSpaceWellheadList();
        Map<String, List<String>> plantIdsByLineIdMap = relation.getPlantIdsByLineIdMap();
        Map<String, List<DynamicQueryBasePowerLineStorageViewOutput>> storageAvgMap = relation.getStorageAvgMap();
        //取年份
        int year = date.year();
        //月份要加一
        int monthIndex = date.month();
        int monthNum = monthIndex + 1;
        int lastMonthNum = monthIndex;
        int lastYear = year;
        if (monthIndex == 0) {
            lastMonthNum = 12;
            lastYear = year - 1;
        }
        String month = BaseUtils.getMonthString(lastMonthNum);
        String lastMonth = BaseUtils.getMonthString(date.month());
        SpaceInstitutionDurationMapper durationMapper = context.getBean(SpaceInstitutionDurationMapper.class);
        List<SpaceOptimizePeriodDTO> periodDTOList = new ArrayList<>(32);
        List<SpaceOptimizeWellheadDTO> wellheadDTOList = new ArrayList<>(64);
        List<SpaceOptimizeDurationDTO> durationDTOList = new ArrayList<>(128);
        List<SpaceOptimizeDurationDTO> unOptimizeDurationList = new ArrayList<>(128);
        String executionCycleForMonth = BaseUtils.getExecutionCycleForMonth(date);
        DateTime optimizeDeadline = DateUtil.endOfMonth(date);
        List<DynamicQueryPlantPredictedPowerOutput> avgPowerList;
        //优化
        for (SpaceInstitutionDetailEnt detail : detailEntList) {
            String detailId = detail.getId();
            String lineId = detail.getLineId();
            //创建记录
            String periodId = super.createOptimizePeriod(periodDTOList, detailId, lineId, executionCycleForMonth, optimizeDeadline);
            //获取当前制度对应的光伏预测数据列表
            avgPowerList = super.getAveragePowerGenerationListByPlantIds(context,
                    DynamicQueryPlantPredictedPowerInput.builder()
                            .plantIds(plantIdsByLineIdMap.get(lineId))
                            .yearTime(String.valueOf(year))
                            .monthTime(month)
                            .build()
            );
            switch (detail.getGridTypeKey()) {
                //并网型优化
                case "1":
                    super.gridConnectedOptimization(context, durationMapper, wellheadDTOList, durationDTOList, unOptimizeDurationList,
                            spaceWellheadList, avgPowerList, detail, monthNum, detailId, periodId, lineId);
                    break;
                //离网型优化
                case "0":
                    //根据类型过滤井口:大间开,连抽井不优化
                    List<SpaceInstitutionWellheadView> wellheadViewList = this.getWellheadViewList(spaceWellheadList, detailId);
                    if (CollUtil.isEmpty(wellheadViewList)) {
                        continue;
                    }
                    //获取井口间开时间段
                    Map<String, List<SpaceInstitutionDurationEnt>> durationMap = this.getDurationMap(durationMapper, detailId);
                    if (CollUtil.isEmpty(durationMap)) {
                        //没有设置时间段,无法优化
                        continue;
                    }
                    //时间差
                    int between = 0;
                    //启动间隔累积
                    int startInterval = 0;
                    //判断是否第一口井
                    boolean isFirstWellhead;
                    //第一口井启动时间
                    DateTime firstStartTime = null;
                    //第一口井开井时间
                    int firstIndex = 0;
                    //井口累积运行总功率
                    BigDecimal totalOperatingPower = BigDecimal.ZERO;
                    //离网时间段
                    List<SpaceOptimizeDurationDTO> offGridPeriodList = new ArrayList<>(32);
                    for (int w = 0, wellheadSize = wellheadViewList.size(); w < wellheadSize; w++) {
                        SpaceInstitutionWellheadView wellhead = wellheadViewList.get(w);
                        String wellheadId = wellhead.getWellheadId();
                        String recordId = this.createOptimizeWellhead(wellheadDTOList, periodId, wellheadId, wellhead.getWellNumber());
                        //取当前井口最大发电量
                        BigDecimal serviceRating = wellhead.getServiceRating();
                        List<SpaceInstitutionDurationEnt> durationConfigList = durationMap.get(wellhead);
                        if (CollUtil.isEmpty(durationConfigList)) {
                            //没有设置时间段,无法优化
                            continue;
                        }
                        //保存原始记录
                        for (SpaceInstitutionDurationEnt durationEnt : durationConfigList) {
                            this.createUnOptimizeDuration(unOptimizeDurationList, durationEnt, periodId, recordId, wellheadId);
                        }
                        totalOperatingPower = totalOperatingPower.add(serviceRating);
                        if (w == 0) {
                            //第一个井口
                            isFirstWellhead = true;
                        } else {
                            //累加
                            startInterval = startInterval + detail.getStartInterval();
                            isFirstWellhead = false;
                        }
                        for (int d = 0, durationSize = durationConfigList.size(); d < durationSize; d++) {
                            SpaceInstitutionDurationEnt duration = durationConfigList.get(d);
                            String openWellTime = duration.getOpenWellTime();
                            DateTime startTime = DateUtil.parse(openWellTime + BusinessConstant.INITIALIZATION_SECOND, BusinessConstant.TIME_FORMAT);
                            DateTime endTime = DateUtil.parse(duration.getCloseWellTime() + BusinessConstant.INITIALIZATION_SECOND, BusinessConstant.TIME_FORMAT);
                            //第一次启动
                            if (d == 0) {
                                //第一口井的启动时间
                                if (isFirstWellhead) {
                                    firstStartTime = startTime;
                                    //计算开井时间
                                    for (int a = 0, avgPowerSize = avgPowerList.size(); a < avgPowerSize; a++) {
                                        DynamicQueryPlantPredictedPowerOutput predictedPower = avgPowerList.get(a);
                                        //当日时间段平均光伏出力>=第一口井运行负荷时,该时间为第一口井运行时间
                                        if (predictedPower.getPower().compareTo(serviceRating) >= BusinessConstant.ZERO) {
                                            firstIndex = a;
                                            break;
                                        }
                                    }
                                    DynamicQueryPlantPredictedPowerOutput start = avgPowerList.get(firstIndex);
                                    String startTimeString = start.getHourTime() + start.getMinTime();
                                    //计算第一次关井时间,按照间开时间段顺延
                                    int startDuration = (int) startTime.between(endTime, DateUnit.MINUTE);
                                    DateTime endTimeOptimize = startTime.offset(DateField.MINUTE, startDuration);
                                    this.createDurationOffGridPeriod(offGridPeriodList, wellheadId, startTimeString, endTimeOptimize.toString(BusinessConstant.MINUTES_FORMAT));
                                    //计算时间偏移
                                    DateTime startTimeOptimize = DateUtil.parse(startTimeString, BusinessConstant.TIME_FORMAT);
                                    //取时间间隔(分钟)
                                    between = BaseUtils.getTimeDifferenceMinute(startTimeOptimize, startTime);
                                }
                                //其它井口的第一次启动时间
                                else {
                                    DateTime startTimeOffset = firstStartTime.offset(DateField.MINUTE, startInterval);
                                    int startIndex = -1;
                                    for (int a = 0, avgPowerSize = avgPowerList.size(); a < avgPowerSize; a++) {
                                        DynamicQueryPlantPredictedPowerOutput predictedPower = avgPowerList.get(a);
                                        if (DateUtil.parse(predictedPower.getHourTime() + predictedPower.getMinTime(), BusinessConstant.TIME_FORMAT)
                                                .compareTo(startTimeOffset) >= 0) {
                                            //判断第一口井启动时间+启动间隔时日平均光伏出力-前两口井的运行功率是否为正数
                                            if (predictedPower.getPower().compareTo(totalOperatingPower) >= 0) {
                                                //确定第二口井第一次开井时间为第一口井启动时间+启动间隔
                                                startIndex = a;
                                                break;
                                            }
                                        }
                                    }
                                    if (startIndex > -1) {
                                        DynamicQueryPlantPredictedPowerOutput start = avgPowerList.get(startIndex);
                                        DateTime startTimeOptimize = DateUtil.parse(start.getHourTime() + start.getMinTime(), BusinessConstant.TIME_FORMAT);
                                        //计算未优化启动间隔
                                        int openDuration = (int) startTime.between(endTime, DateUnit.MINUTE);
                                        DateTime endTimeOptimize = startTimeOptimize.offset(DateField.MINUTE, openDuration);
                                        this.createDurationOffGridPeriod(offGridPeriodList, wellheadId, openWellTime, BaseUtils.getEndTimeString(endTimeOptimize));
                                        //取时间间隔(分钟)
                                        between = BaseUtils.getTimeDifferenceMinute(endTimeOptimize, endTime);
                                    } else {
                                        //无法优化
                                    }
                                }
                            } else {
                                DateTime offset = startTime.offset(DateField.MINUTE, between);
                                if (offset.compareTo(BusinessConstant.DATE_FLAG) > 0) {
                                    //如果时间超过当天,舍弃
                                    continue;
                                }
                                //计算偏移
                                this.createDurationOffGridPeriod(offGridPeriodList, wellheadId,
                                        offset.toString(BusinessConstant.MINUTES_FORMAT),
                                        BaseUtils.getEndTimeString(DateUtil.parse(duration.getCloseWellTime() + BusinessConstant.INITIALIZATION_SECOND, BusinessConstant.TIME_FORMAT)
                                                .offset(DateField.MINUTE, between))
                                );
                            }
                        }
                    }
                    //离网优化
                    if (CollUtil.isNotEmpty(offGridPeriodList)) {
                        if (!storageAvgMap.containsKey(lineId)) {
                            //没有储能设备,无法计算
                            continue;
                        }
                        //截取从第一次开井时间往后的时间段
                        List<DynamicQueryPlantPredictedPowerOutput> subAvgPowerList = CollUtil.sub(avgPowerList, firstIndex + 1, avgPowerList.size());
                        if (CollUtil.isEmpty(subAvgPowerList)) {
                            //没有时间段,无法计算
                            continue;
                        }
                        //查获取前一个月该时间点储能剩余电量，求平均数
                        List<DynamicQueryStoragePredictedPowerOutput> averageEnergyStorageList = super.getAverageEnergyStorageListByParam(context,
                                DynamicQueryStoragePredictedPowerInput.builder()
                                        .storageIds(plantIdsByLineIdMap.get(lineId))
                                        .yearTime(String.valueOf(lastYear))
                                        .monthTime(lastMonth)
                                        .build()
                        );
                        if (CollUtil.isEmpty(averageEnergyStorageList)) {
                            //没有储能发电量,无法计算
                            continue;
                        }
                        //取储能计算参数
                        List<DynamicQueryBasePowerLineStorageViewOutput> avgStorageViewList = storageAvgMap.get(lineId);
                        DynamicQueryBasePowerLineStorageViewOutput avgStorageView = avgStorageViewList.get(0);
                        //平均额定放电效率(%)
                        BigDecimal avgRatedDischargeEfficiency = avgStorageView.getRatedDischargeEfficiency();
                        //平均额定放电深度(%)
                        BigDecimal avgRatedDischargeDepth = avgStorageView.getRatedDischargeDepth();
                        //平均额定放电功率(KW)
                        BigDecimal avgRatedDischargePower = avgStorageView.getRatedDischargePower();
                        //逐15分钟计算光伏出力-井场运行功率之和，获得光伏出力不足时间段,并计算储能可用时长
                        Iterator<DynamicQueryStoragePredictedPowerOutput> iterator = averageEnergyStorageList.iterator();
                        for (DynamicQueryPlantPredictedPowerOutput avg : subAvgPowerList) {
                            if (avg.getPower().compareTo(totalOperatingPower) >= 0) {
                                String hourTime = avg.getHourTime();
                                String minTime = avg.getMinTime();
                                while (iterator.hasNext()) {
                                    DynamicQueryStoragePredictedPowerOutput storagePredicted = iterator.next();
                                    if (StringUtils.equals(hourTime, storagePredicted.getHourTime()) && StringUtils.equals(minTime, storagePredicted.getMinTime())) {
                                        //计算储能可供电时长:（储能剩余容量*放电深度*放电效率）/额定放电功率
                                        avg.setPowerDuration(
                                                storagePredicted.getPower()
                                                        .multiply(avgRatedDischargeDepth)
                                                        .multiply(avgRatedDischargeEfficiency)
                                                        .divide(avgRatedDischargePower)
                                                        .setScale(BusinessConstant.ZERO, RoundingMode.DOWN)
                                                        .intValueExact()
                                        );
                                        break;
                                    }
                                    //删除计算过的时间段
                                    iterator.remove();
                                }
                            }
                        }
                        //拆分时间段
                        for (SpaceOptimizeDurationDTO durationDTO : offGridPeriodList) {

                        }
                    }
                    break;
                default:
                    //电网类型不存在
            }
        }
        //开启事务
        return XTransactionHelper.begin(context, () -> {
            int size;
            SpaceOptimizeViewMapper optimizeViewMapper = context.getBean(SpaceOptimizeViewMapper.class);
            if (CollUtil.isNotEmpty(periodDTOList)) {
                size = periodDTOList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizePeriodDTO>> subList = BaseUtils.getSubList(periodDTOList);
                    subList.forEach(b -> optimizeViewMapper.longPeriodBatchInsertList(b));
                } else {
                    optimizeViewMapper.longPeriodBatchInsertList(periodDTOList);
                }
            }
            if (CollUtil.isNotEmpty(wellheadDTOList)) {
                size = wellheadDTOList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizeWellheadDTO>> subList = BaseUtils.getSubList(wellheadDTOList);
                    subList.forEach(b -> optimizeViewMapper.longWellheadBatchInsertList(b));
                } else {
                    optimizeViewMapper.longWellheadBatchInsertList(wellheadDTOList);
                }
            }
            if (CollUtil.isNotEmpty(unOptimizeDurationList)) {
                size = unOptimizeDurationList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizeDurationDTO>> subList = BaseUtils.getSubList(unOptimizeDurationList);
                    subList.forEach(b -> optimizeViewMapper.longDurationBatchInsertList(b));
                } else {
                    optimizeViewMapper.longDurationBatchInsertList(unOptimizeDurationList);
                }
            }
            if (CollUtil.isNotEmpty(durationDTOList)) {
                size = durationDTOList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizeDurationDTO>> subList = BaseUtils.getSubList(durationDTOList);
                    subList.forEach(b -> optimizeViewMapper.longDurationBatchInsertList(b));
                } else {
                    optimizeViewMapper.longDurationBatchInsertList(durationDTOList);
                }
            }
            return XServiceResult.OK;
        });
    }

    /*-----------------------------------private-----------------------------------*/


    public void createDurationOffGridPeriod(List<SpaceOptimizeDurationDTO> offGridPeriodList, String wellheadId,
                                            String openWellTime, String endTimeString) {
        SpaceOptimizeDurationDTO durationDTO = new SpaceOptimizeDurationDTO();
        durationDTO.setWellheadId(wellheadId);
        durationDTO.setOpenWellTime(openWellTime);
        durationDTO.setCloseWellTime(endTimeString);
        offGridPeriodList.add(durationDTO);
    }
}