package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import pps.cloud.prediction.service.data.plant_predicted_power_data.DynamicQueryPlantPredictedPowerInput;
import pps.cloud.prediction.service.data.plant_predicted_power_data.DynamicQueryPlantPredictedPowerOutput;
import pps.cloud.space.service.ISpaceOptimizeMidCloudService;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.utils.BaseUtils;
import pps.core.space.entity.*;
import pps.core.space.mapper.SpaceInstitutionDurationMapper;
import pps.core.space.mapper.SpaceOptimizeViewMapper;
import pps.core.space.service.data.SpaceOptimizeLineRelation;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.data.XServiceResult;
import xstartup.helper.XTransactionHelper;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 中短期间开优化Cloud模块
 *
 * @author ZWT
 * @date 2023/09/18 14:14
 */
@XService
public class SpaceOptimizeMidCloudServiceImpl extends SpaceOptimizeBaseService implements ISpaceOptimizeMidCloudService {

    /**
     * 中短期间开优化Cloud模块--定时任务
     *
     * @param context 上下文
     * @return {@link XServiceResult}
     */
    @Override
    public XServiceResult optimizeMidJob(XContext context) {
        //取当前时间
        DateTime date = DateUtil.date();
        String startWeek = date.toString(BusinessConstant.DATE_FORMAT_DAY);
        //取生效中的基础间开
        List<SpaceInstitutionDetailEnt> detailEntList = super.getEffectiveSpaceInstitution(context, startWeek);
        SpaceOptimizeLineRelation wellheadAndPlant = super.getWellheadAndPlant(context, detailEntList);
        List<SpaceInstitutionWellheadView> spaceWellheadList = wellheadAndPlant.getSpaceWellheadList();
        Map<String, List<String>> plantIdsByLineIdMap = wellheadAndPlant.getPlantIdsByLineIdMap();
        //月份要加一
        int monthNum = date.month() + 1;
        SpaceInstitutionDurationMapper durationMapper = context.getBean(SpaceInstitutionDurationMapper.class);
        List<SpaceOptimizePeriodDTO> periodDTOList = new ArrayList<>(32);
        List<SpaceOptimizeWellheadDTO> wellheadDTOList = new ArrayList<>(64);
        List<SpaceOptimizeDurationDTO> durationDTOList = new ArrayList<>(128);
        List<SpaceOptimizeDurationDTO> unOptimizeDurationList = new ArrayList<>(128);
        String executionCycleForWeek = BaseUtils.getExecutionCycleForWeek(date);
        //优化截止日期
        DateTime optimizeDeadline = DateUtil.endOfWeek(date);
        //日期查询范围结束条件,下周一
        String nextWeek = DateUtil.beginOfWeek(DateUtil.nextWeek()).toString(BusinessConstant.DATE_FORMAT_DAY);
        //优化
        for (SpaceInstitutionDetailEnt detail : detailEntList) {
            String detailId = detail.getId();
            String lineId = detail.getLineId();
            //创建记录
            String periodId = super.createOptimizePeriod(periodDTOList, detailId, lineId, executionCycleForWeek, optimizeDeadline);
            switch (detail.getGridTypeKey()) {
                //并网型优化
                case "1":
                    //获取当前制度对应的光伏预测数据列表
                    List<DynamicQueryPlantPredictedPowerOutput> avgPowerList = super.getAveragePowerGenerationListByPlantIds(context,
                            DynamicQueryPlantPredictedPowerInput.builder()
                                    .plantIds(plantIdsByLineIdMap.get(lineId))
                                    .startTime(startWeek)
                                    .endTime(nextWeek)
                                    .build()
                    );
                    super.gridConnectedOptimization(context, durationMapper, wellheadDTOList, durationDTOList, unOptimizeDurationList,
                            spaceWellheadList, avgPowerList, detail, monthNum, detailId, periodId, lineId);
                    break;
                //离网型优化
                case "0":
                    break;
                default:
                    //电网类型不存在
            }
        }
        //开启事务
        return XTransactionHelper.begin(context, () -> {
            int size;
            SpaceOptimizeViewMapper optimizeViewMapper = context.getBean(SpaceOptimizeViewMapper.class);
            if (CollUtil.isNotEmpty(periodDTOList)) {
                size = periodDTOList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizePeriodDTO>> subList = BaseUtils.getSubList(periodDTOList);
                    subList.forEach(b -> optimizeViewMapper.midPeriodBatchInsertList(b));
                } else {
                    optimizeViewMapper.midPeriodBatchInsertList(periodDTOList);
                }
            }
            if (CollUtil.isNotEmpty(wellheadDTOList)) {
                size = wellheadDTOList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizeWellheadDTO>> subList = BaseUtils.getSubList(wellheadDTOList);
                    subList.forEach(b -> optimizeViewMapper.midWellheadBatchInsertList(b));
                } else {
                    optimizeViewMapper.midWellheadBatchInsertList(wellheadDTOList);
                }
            }
            if (CollUtil.isNotEmpty(unOptimizeDurationList)) {
                size = unOptimizeDurationList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizeDurationDTO>> subList = BaseUtils.getSubList(unOptimizeDurationList);
                    subList.forEach(b -> optimizeViewMapper.midDurationBatchInsertList(b));
                } else {
                    optimizeViewMapper.midDurationBatchInsertList(unOptimizeDurationList);
                }
            }
            if (CollUtil.isNotEmpty(durationDTOList)) {
                size = durationDTOList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizeDurationDTO>> subList = BaseUtils.getSubList(durationDTOList);
                    subList.forEach(b -> optimizeViewMapper.midDurationBatchInsertList(b));
                } else {
                    optimizeViewMapper.midDurationBatchInsertList(durationDTOList);
                }
            }
            return XServiceResult.OK;
        });
    }

    /*-----------------------------------private-----------------------------------*/
}