package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import pps.core.space.entity.SpaceInstitutionDetailEnt;
import pps.core.space.entity.SpaceOptimizeShortDurationView;
import pps.core.space.entity.SpaceOptimizeShortPeriodEnt;
import pps.core.space.entity.SpaceOptimizeShortWellheadEnt;
import pps.core.space.mapper.SpaceInstitutionDetailMapper;
import pps.core.space.mapper.SpaceOptimizeShortDurationViewMapper;
import pps.core.space.mapper.SpaceOptimizeShortPeriodMapper;
import pps.core.space.mapper.SpaceOptimizeShortWellheadMapper;
import pps.core.space.service.data.SpaceOptimizeDateDuration;
import pps.core.space.service.data.space_optimize_short_period.GetSpaceOptimizeShortPeriodViewInput;
import pps.core.space.service.data.space_optimize_short_period.GetSpaceOptimizeShortPeriodViewOutput;
import pps.core.space.service.data.space_optimize_short_wellhead.GetSpaceOptimizeShortWellheadViewOutput;
import pps.core.space.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XSingleResult;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 短期间开优化模块
 *
 * @author ZWT
 * @date 2023/09/07
 */
@XService
@XText("短期间开优化模块")
public class SpaceOptimizeShortPeriodService {

    /**
     * 短期间开优化--详情
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetSpaceOptimizeShortPeriodViewOutput}>
     */
    @XApiAnonymous
    @XApiGet
    @XText("短期间开优化--详情")
    public XSingleResult<GetSpaceOptimizeShortPeriodViewOutput> getSpaceOptimizeShortPeriodView(XContext context, GetSpaceOptimizeShortPeriodViewInput input) {
        SpaceOptimizeShortPeriodMapper mapper = context.getBean(SpaceOptimizeShortPeriodMapper.class);
        //查当日记录
        SpaceOptimizeShortPeriodEnt period = mapper.selectOne(
                new LambdaQueryWrapper<SpaceOptimizeShortPeriodEnt>()
                        .eq(SpaceOptimizeShortPeriodEnt::getLineId, input.getLineId())
                        .apply("DATE_FORMAT( optimize_date, '%Y-%m-%d' ) = curdate()")
        );
        GetSpaceOptimizeShortPeriodViewOutput output = new GetSpaceOptimizeShortPeriodViewOutput();
        if (Objects.nonNull(period)) {
            //查间开制度详情
            SpaceInstitutionDetailMapper detailMapper = context.getBean(SpaceInstitutionDetailMapper.class);
            SpaceInstitutionDetailEnt detailEnt = detailMapper.selectById(period.getInstitutionId());
            output.setGridTypeKey(detailEnt.getGridTypeKey());
            String shortPeriodId = period.getId();
            //查井口列表
            SpaceOptimizeShortWellheadMapper wellheadMapper = context.getBean(SpaceOptimizeShortWellheadMapper.class);
            List<SpaceOptimizeShortWellheadEnt> wellheadList = wellheadMapper.selectList(new LambdaQueryWrapper<SpaceOptimizeShortWellheadEnt>()
                    .eq(SpaceOptimizeShortWellheadEnt::getShortPeriodId, shortPeriodId)
            );
            if (CollUtil.isNotEmpty(wellheadList)) {
                List<GetSpaceOptimizeShortWellheadViewOutput> wellheadViewOutputs = XCopyUtils.copyNewList(wellheadList, GetSpaceOptimizeShortWellheadViewOutput.class);
                //查井口间隔配置
                SpaceOptimizeShortDurationViewMapper durationViewMapper = context.getBean(SpaceOptimizeShortDurationViewMapper.class);
                List<SpaceOptimizeShortDurationView> durationViewList = durationViewMapper.selectList(SpaceOptimizeShortDurationView.builder()
                        .shortPeriodId(shortPeriodId)
                        .build()
                );
                if (CollUtil.isNotEmpty(durationViewList)) {
                    Map<String, List<SpaceOptimizeDateDuration>> collect = XCopyUtils.copyNewList(durationViewList, SpaceOptimizeDateDuration.class)
                            .stream()
                            .collect(Collectors.groupingBy(SpaceOptimizeDateDuration::getWellheadId));
                    wellheadViewOutputs.forEach(w -> {
                        if (collect.containsKey(w.getWellheadId())) {
                            w.setDurationList(ServiceUtil.getAllDateDuration(collect.get(w.getWellheadId())));
                        } else {
                            w.setDurationList(new ArrayList<>(0));
                        }
                    });
                }
                output.setWellheadList(wellheadViewOutputs);
            }
        } else {
            output.setWellheadList(new ArrayList<>(0));
        }
        return XSingleResult.success(output);
    }
}