package pps.core.prediction.service;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.apache.commons.lang3.StringUtils;
import pps.cloud.prediction.service.IPlantPredictedPowerCloudService;
import pps.cloud.prediction.service.data.plant_predicted_power_data.DynamicQueryPlantPredictedPowerInput;
import pps.cloud.prediction.service.data.plant_predicted_power_data.DynamicQueryPlantPredictedPowerOutput;
import pps.core.prediction.entity.PlantPredictedPowerDataEnt;
import pps.core.prediction.mapper.PlantPredictedPowerDataMapper;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;

import java.util.ArrayList;
import java.util.List;

/**
 * 光伏预测Cloud模块
 *
 * @author ZWT
 * @date 2023/09/13 19:05
 */
@XService
public class PlantPredictedPowerCloudServiceImpl implements IPlantPredictedPowerCloudService {

    /**
     * 光伏预测Cloud模块--条件查询时段发电量平均值
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryPlantPredictedPowerOutput}>
     */
    @Override
    public XListResult<DynamicQueryPlantPredictedPowerOutput> queryAveragePowerGenerationListByParam(XContext context, DynamicQueryPlantPredictedPowerInput input) {
        String plantId = input.getPlantId();
        List<String> plantIds = input.getPlantIds();
        String yearTime = input.getYearTime();
        String monthTime = input.getMonthTime();
        String startTime = input.getStartTime();
        String endTime = input.getEndTime();
        PlantPredictedPowerDataMapper mapper = context.getBean(PlantPredictedPowerDataMapper.class);
        List<PlantPredictedPowerDataEnt> list = mapper.selectList(
                new QueryWrapper<PlantPredictedPowerDataEnt>()
                        .select("year_time",
                                "month_time",
                                "hour_time",
                                "min_time",
                                "AVG( power ) AS power")
                        .lambda()
                        .eq(StringUtils.isNotBlank(plantId), PlantPredictedPowerDataEnt::getPlantId, plantId)
                        .in(CollUtil.isNotEmpty(plantIds), PlantPredictedPowerDataEnt::getPlantId, plantIds)
                        .eq(StringUtils.isNotBlank(yearTime), PlantPredictedPowerDataEnt::getYearTime, yearTime)
                        .eq(StringUtils.isNotBlank(monthTime), PlantPredictedPowerDataEnt::getMonthTime, monthTime)
                        .between(!StringUtils.isAnyBlank(startTime, endTime), PlantPredictedPowerDataEnt::getDataDate, startTime, endTime)
                        .groupBy(PlantPredictedPowerDataEnt::getYearTime,
                                PlantPredictedPowerDataEnt::getMonthTime,
                                PlantPredictedPowerDataEnt::getHourTime,
                                PlantPredictedPowerDataEnt::getMinTime)
        );
        List<DynamicQueryPlantPredictedPowerOutput> outputs;
        if (CollUtil.isEmpty(list)) {
            outputs = new ArrayList<>(0);
        } else {
            outputs = XCopyUtils.copyNewList(list, DynamicQueryPlantPredictedPowerOutput.class);
        }
        return XListResult.success(outputs);
    }
}
